@@ -8,6 +8,11 @@ mod nettle;
 #[cfg(feature = "crypto-nettle")]
 pub use self::nettle::*;
 
+#[cfg(feature = "crypto-rust")]
+mod rust;
+#[cfg(feature = "crypto-rust")]
+pub use self::rust::*;
+
 #[cfg(feature = "crypto-cng")]
 mod cng;
 #[cfg(feature = "crypto-cng")]
