@@ -42,7 +42,7 @@ impl Mode for KeyWrapper {
         let missing = (block_size - (src.len() % block_size)) % block_size;
         let src = if missing != 0 {
             _src = vec![0u8; src.len() + missing];
-            &mut _src[..src.len()].copy_from_slice(src);
+            _src[..src.len()].copy_from_slice(src);
             &_src
         } else {
             src
@@ -68,7 +68,7 @@ impl Mode for KeyWrapper {
         let missing = (block_size - (src.len() % block_size)) % block_size;
         let src = if missing != 0 {
             _src = vec![0u8; src.len() + missing];
-            &mut _src[..src.len()].copy_from_slice(src);
+            _src[..src.len()].copy_from_slice(src);
             &_src
         } else {
             src
@@ -134,8 +134,9 @@ impl SymmetricAlgorithm {
         }
     }
 
-    /// Length of a key for this algorithm in bytes.  Fails if the crypto
-    /// backend does not support this algorithm.
+    /// Length of a key for this algorithm in bytes.
+    ///
+    /// Fails if the crypto backend does not support this algorithm.
     pub fn key_size(self) -> Result<usize> {
         Ok(match self {
             SymmetricAlgorithm::TripleDES => 24,
@@ -146,8 +147,9 @@ impl SymmetricAlgorithm {
         })
     }
 
-    /// Length of a block for this algorithm in bytes.  Fails if the crypto
-    /// backend does not support this algorithm.
+    /// Length of a block for this algorithm in bytes.
+    ///
+    /// Fails if the crypto backend does not support this algorithm.
     pub fn block_size(self) -> Result<usize> {
         Ok(match self {
             SymmetricAlgorithm::TripleDES => 8,
