@@ -117,6 +117,16 @@ impl MPI {
         val
     }
 
+    /// Creates a new MPI representing zero.
+    pub fn zero() -> Self {
+        Self::new(&[])
+    }
+
+    /// Tests whether the MPI represents zero.
+    pub fn is_zero(&self) -> bool {
+        self.value().is_empty()
+    }
+
     /// Returns the length of the MPI in bits.
     ///
     /// Leading zero-bits are not included in the returned size.
@@ -157,7 +167,8 @@ impl MPI {
     ///
     /// Returns `Error::UnsupportedEllipticCurve` if the curve is not
     /// supported, `Error::MalformedMPI` if the point is formatted
-    /// incorrectly.
+    /// incorrectly, `Error::InvalidOperation` if the given curve is
+    /// operating on native octet strings.
     pub fn decode_point(&self, curve: &Curve) -> Result<(&[u8], &[u8])> {
         Self::decode_point_common(self.value(), curve)
     }
@@ -190,8 +201,13 @@ impl MPI {
                 Ok((&value[1..], &[]))
             },
 
-            _ => {
-
+            NistP256
+                | NistP384
+                | NistP521
+                | BrainpoolP256
+                | BrainpoolP512
+                =>
+            {
                 // Length of one coordinate in bytes, rounded up.
                 let coordinate_length = (curve.len()? + 7) / 8;
 
@@ -216,6 +232,9 @@ impl MPI {
                 Ok((&value[1..1 + coordinate_length],
                     &value[1 + coordinate_length..]))
             },
+
+            Unknown(_) =>
+                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
         }
     }
 
@@ -423,7 +442,8 @@ impl ProtectedMPI {
     ///
     /// Returns `Error::UnsupportedEllipticCurve` if the curve is not
     /// supported, `Error::MalformedMPI` if the point is formatted
-    /// incorrectly.
+    /// incorrectly, `Error::InvalidOperation` if the given curve is
+    /// operating on native octet strings.
     pub fn decode_point(&self, curve: &Curve) -> Result<(&[u8], &[u8])> {
         MPI::decode_point_common(self.value(), curve)
     }
@@ -491,7 +511,7 @@ pub enum PublicKey {
         y: MPI,
     },
 
-    /// DJBs "Twisted" Edwards curve DSA public key.
+    /// DJB's "Twisted" Edwards curve DSA public key.
     EdDSA {
         /// Curve we're using. Must be curve 25519.
         curve: Curve,
@@ -499,7 +519,7 @@ pub enum PublicKey {
         q: MPI,
     },
 
-    /// NISTs Elliptic curve DSA public key.
+    /// NIST's Elliptic Curve DSA public key.
     ECDSA {
         /// Curve we're using.
         curve: Curve,
@@ -507,15 +527,15 @@ pub enum PublicKey {
         q: MPI,
     },
 
-    /// Elliptic curve ElGamal public key.
+    /// Elliptic Curve Diffie-Hellman public key.
     ECDH {
         /// Curve we're using.
         curve: Curve,
         /// Public point.
         q: MPI,
-        /// Hash algorithm used for key derivation.
+        /// Algorithm used to derive the Key Encapsulation Key.
         hash: HashAlgorithm,
-        /// Algorithm used w/the derived key.
+        /// Algorithm used to encapsulate the session key.
         sym: SymmetricAlgorithm,
     },
 
@@ -663,19 +683,19 @@ pub enum SecretKeyMaterial {
         x: ProtectedMPI,
     },
 
-    /// DJBs "Twisted" Edwards curve DSA secret key.
+    /// DJB's "Twisted" Edwards curve DSA secret key.
     EdDSA {
         /// Secret scalar.
         scalar: ProtectedMPI,
     },
 
-    /// NISTs Elliptic curve DSA secret key.
+    /// NIST's Elliptic Curve DSA secret key.
     ECDSA {
         /// Secret scalar.
         scalar: ProtectedMPI,
     },
 
-    /// Elliptic curve ElGamal secret key.
+    /// Elliptic Curve Diffie-Hellman public key.
     ECDH {
         /// Secret scalar.
         scalar: ProtectedMPI,
