@@ -243,8 +243,15 @@ impl S2K {
     /// Returns whether this S2K mechanism is supported.
     pub fn is_supported(&self) -> bool {
         use self::S2K::*;
-        #[allow(deprecated)] {
-            matches!(self, Simple { .. } | Salted { .. } | Iterated { .. })
+        #[allow(deprecated)]
+        match self {
+            Simple { .. }
+            | Salted { .. }
+            | Iterated { .. }
+            => true,
+            S2K::Private { .. }
+            | S2K::Unknown { .. }
+            => false,
         }
     }
 
