@@ -140,6 +140,8 @@ impl From<&Fingerprint> for KeyID {
         match fp {
             Fingerprint::V4(fp) =>
                 KeyID::from_bytes(&fp[fp.len() - 8..]),
+            Fingerprint::V5(fp) =>
+                KeyID::Invalid(fp.iter().cloned().collect()),
             Fingerprint::Invalid(fp) => {
                 KeyID::Invalid(fp.clone())
             }
@@ -152,6 +154,8 @@ impl From<Fingerprint> for KeyID {
         match fp {
             Fingerprint::V4(fp) =>
                 KeyID::from_bytes(&fp[fp.len() - 8..]),
+            Fingerprint::V5(fp) =>
+                KeyID::Invalid(fp.into()),
             Fingerprint::Invalid(fp) => {
                 KeyID::Invalid(fp)
             }
