@@ -6,14 +6,182 @@ use std::fmt;
 use buffered_reader::BufferedReader;
 use buffered_reader::buffered_reader_generic_read_impl;
 
-use crate::{
-    Result,
-    types::HashAlgorithm,
-};
-use crate::parse::{Cookie, HashesFor, Hashing, HashingMode};
+use crate::crypto::hash::Digest;
+use crate::parse::{Cookie, HashesFor, Hashing};
+use crate::Result;
+use crate::types::HashAlgorithm;
+use crate::types::SignatureType;
 
 const TRACE : bool = false;
 
+/// Controls line-ending normalization during hashing.
+///
+/// OpenPGP normalizes line endings when signing or verifying text
+/// signatures.
+#[derive(Clone, Eq)]
+pub(crate) enum HashingMode<T> {
+    /// Hash for a binary signature.
+    ///
+    /// The data is hashed as-is.
+    Binary(T),
+
+    /// Hash for a text signature.
+    ///
+    /// The data is hashed with line endings normalized to `\r\n`.
+    Text(T),
+
+    /// Like Text, but the last character that we hashed was a '\r'
+    /// that we converted to a '\r\n'.
+    TextLastWasCr(T),
+}
+
+impl<T: std::fmt::Debug> std::fmt::Debug for HashingMode<T> {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => write!(f, "Binary({:?})", t),
+            Text(t) => write!(f, "Text({:?})", t),
+            TextLastWasCr(t) => write!(f, "Text(last was CR, {:?})", t),
+        }
+    }
+}
+
+impl<T: PartialEq> PartialEq for HashingMode<T> {
+    fn eq(&self, other: &Self) -> bool {
+        use self::HashingMode::*;
+        match (self, other) {
+            (Binary(s), Binary(o)) => s.eq(o),
+
+            (Text(s), Text(o)) => s.eq(o),
+            (TextLastWasCr(s), Text(o)) => s.eq(o),
+            (Text(s), TextLastWasCr(o)) => s.eq(o),
+            (TextLastWasCr(s), TextLastWasCr(o)) => s.eq(o),
+
+            _ => false,
+        }
+    }
+}
+
+impl<T> HashingMode<T> {
+    pub(crate) fn map<U, F: Fn(&T) -> U>(&self, f: F) -> HashingMode<U> {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => Binary(f(t)),
+            Text(t) => Text(f(t)),
+            TextLastWasCr(t) => TextLastWasCr(f(t)),
+        }
+    }
+
+    pub(crate) fn as_ref(&self) -> &T {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => t,
+            Text(t) => t,
+            TextLastWasCr(t) => t,
+        }
+    }
+
+    pub(crate) fn as_mut(&mut self) -> &mut T {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => t,
+            Text(t) => t,
+            TextLastWasCr(t) => t,
+        }
+    }
+
+    pub(crate) fn for_signature(t: T, typ: SignatureType) -> Self {
+        if typ == SignatureType::Text {
+            HashingMode::Text(t)
+        } else {
+            HashingMode::Binary(t)
+        }
+    }
+
+    pub(crate) fn into_inner(self) -> T {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => t,
+            Text(t) => t,
+            TextLastWasCr(t) => t,
+        }
+    }
+}
+
+impl<D> HashingMode<D>
+    where D: Digest + Clone
+{
+    /// Updates the given hash context.  When in text mode, normalize
+    /// the line endings to "\r\n" on the fly.
+    pub(crate) fn update(&mut self, data: &[u8]) {
+        if data.is_empty() {
+            // This isn't just a short circuit.  It preserves
+            // `last_was_cr`, which running through the code would
+            // not.
+            return;
+        }
+
+        let (h, mut last_was_cr) = match self {
+            HashingMode::Text(h) => (h, false),
+            HashingMode::TextLastWasCr(h) => (h, true),
+            HashingMode::Binary(h) => return h.update(data),
+        };
+
+        let mut line = data;
+        let last_is_cr = line.last() == Some(&b'\r');
+        while ! line.is_empty() {
+            let mut next = 0;
+            for (i, c) in line.iter().cloned().enumerate() {
+                match c {
+                    b'\n' if last_was_cr => {
+                        // We already hash the \n.
+                        assert_eq!(i, 0);
+                        assert_eq!(next, 0);
+                        next = 1;
+                        break;
+                    },
+                    b'\r' | b'\n' => {
+                        h.update(&line[..i]);
+                        h.update(b"\r\n");
+                        next = i + 1;
+                        if c == b'\r' && line.get(next) == Some(&b'\n') {
+                            next += 1;
+                        }
+                        break;
+                    },
+                    _ => (),
+                }
+                last_was_cr = false;
+            }
+
+            if next > 0 {
+                line = &line[next..];
+            } else {
+                h.update(line);
+                break;
+            }
+        }
+
+        match (&mut *self, last_is_cr) {
+            (&mut HashingMode::Text(_), false) => {
+                // This is the common case.  Getting a crlf that is
+                // split across two chunks is extremely rare.  Hence,
+                // the clones used to change the variant are rarely
+                // needed.
+            },
+            (&mut HashingMode::Text(ref mut h), true) => {
+                *self = HashingMode::TextLastWasCr(h.clone());
+            }
+            (&mut HashingMode::TextLastWasCr(ref mut h), false) => {
+                *self = HashingMode::Text(h.clone());
+            },
+            (&mut HashingMode::TextLastWasCr(_), true) => (),
+
+            _ => unreachable!("handled above"),
+        }
+    }
+}
+
 pub(crate) struct HashedReader<R: BufferedReader<Cookie>> {
     reader: R,
     cookie: Cookie,
@@ -55,37 +223,6 @@ impl<R: BufferedReader<Cookie>> HashedReader<R> {
     }
 }
 
-/// Updates the given hash context normalizing line endings to "\r\n"
-/// on the fly.
-pub(crate) fn hash_update_text(h: &mut dyn crate::crypto::hash::Digest,
-                               text: &[u8]) {
-    let mut line = text;
-    while ! line.is_empty() {
-        let mut next = 0;
-        for (i, c) in line.iter().cloned().enumerate() {
-            match c {
-                b'\r' | b'\n' => {
-                    h.update(&line[..i]);
-                    h.update(b"\r\n");
-                    next = i + 1;
-                    if c == b'\r' && line.get(next) == Some(&b'\n') {
-                        next += 1;
-                    }
-                    break;
-                },
-                _ => (),
-            }
-        }
-
-        if next > 0 {
-            line = &line[next..];
-        } else {
-            h.update(line);
-            break;
-        }
-    }
-}
-
 impl Cookie {
     fn hash_update(&mut self, data: &[u8]) {
         let level = self.level.unwrap_or(0);
@@ -110,16 +247,13 @@ impl Cookie {
             // We fix that here by hashing the stashed data into the
             // former topmost signature-group's hash.
             assert!(ngroups > 1);
-            for mode in self.sig_groups[ngroups-2].hashes.iter_mut()
+            for h in self.sig_groups[ngroups-2].hashes.iter_mut()
             {
                 t!("({:?}): group {} {:?} hashing {} stashed bytes.",
-                   hashes_for, ngroups-2, mode.map(|ctx| ctx.algo()),
+                   hashes_for, ngroups-2, h.map(|ctx| ctx.algo()),
                    data.len());
 
-                match mode {
-                    HashingMode::Binary(h) => h.update(&stashed_data),
-                    HashingMode::Text(h) => hash_update_text(h, &stashed_data),
-                }
+                h.update(&stashed_data);
             }
         }
 
@@ -142,13 +276,10 @@ impl Cookie {
                 return;
             }
 
-            for mode in sig_group.hashes.iter_mut() {
+            for h in sig_group.hashes.iter_mut() {
                 t!("{:?}: group {} {:?} hashing {} bytes.",
-                   hashes_for, i, mode.map(|ctx| ctx.algo()), data.len());
-                match mode {
-                    HashingMode::Binary(h) => h.update(data),
-                    HashingMode::Text(h) => hash_update_text(h, data),
-                }
+                   hashes_for, i, h.map(|ctx| ctx.algo()), data.len());
+                h.update(data);
             }
         }
     }
@@ -190,17 +321,13 @@ impl Cookie {
 
         // Hash stashed data first.
         if let Some(stashed_data) = self.hash_stash.take() {
-            for mode in self.sig_groups[0].hashes.iter_mut() {
+            for h in self.sig_groups[0].hashes.iter_mut() {
                 t!("{:?}: {:?} hashing {} stashed bytes.",
-                   hashes_for, mode.map(|ctx| ctx.algo()),
+                   hashes_for, h.map(|ctx| ctx.algo()),
                    stashed_data.len());
-                match mode {
-                    HashingMode::Binary(_) =>
-                        unreachable!("CSF transformation uses \
-                                      text signatures"),
-                    HashingMode::Text(h) =>
-                        hash_update_text(h, &stashed_data[..]),
-                }
+                assert!(matches!(h, HashingMode::Text(_)),
+                        "CSF transformation uses text signatures");
+                h.update(&stashed_data[..]);
             }
         }
 
@@ -220,14 +347,12 @@ impl Cookie {
         };
 
         // Hash everything but the last newline now.
-        for mode in self.sig_groups[0].hashes.iter_mut() {
+        for h in self.sig_groups[0].hashes.iter_mut() {
             t!("{:?}: {:?} hashing {} bytes.",
-               hashes_for, mode.map(|ctx| ctx.algo()), l);
-            match mode {
-                HashingMode::Binary(_) =>
-                    unreachable!("CSF transformation uses text signatures"),
-                HashingMode::Text(h) => hash_update_text(h, &data[..l]),
-            }
+               hashes_for, h.map(|ctx| ctx.algo()), l);
+            assert!(matches!(h, HashingMode::Text(_)),
+                    "CSF transformation uses text signatures");
+            h.update(&data[..l]);
         }
 
         // The newline we stash away.  If more text is written
@@ -451,13 +576,20 @@ mod test {
             "one\rtwo\rthree",
             "one\ntwo\r\nthree",
         ] {
-            let mut ctx = HashAlgorithm::SHA256.context()?;
-            super::hash_update_text(&mut ctx, text.as_bytes());
-            let mut digest = vec![0; ctx.digest_size()];
-            let _ = ctx.digest(&mut digest);
-            assert_eq!(
-                &crate::fmt::hex::encode(&digest),
-                "5536758151607BB81CE8D6F49189B2E84763DA9EA84965AB7327E704DAE415EB");
+            for chunk_size in &[ text.len(), 1 ] {
+                let mut ctx
+                    = HashingMode::Text(HashAlgorithm::SHA256.context()?);
+                for chunk in text.as_bytes().chunks(*chunk_size) {
+                    ctx.update(chunk);
+                }
+                let mut ctx = ctx.into_inner();
+                let mut digest = vec![0; ctx.digest_size()];
+                let _ = ctx.digest(&mut digest);
+                assert_eq!(
+                    &crate::fmt::hex::encode(&digest),
+                    "5536758151607BB81CE8D6F49189B2E84763DA9EA84965AB7327E704DAE415EB",
+                    "{:?}, chunk size: {}", text, chunk_size);
+            }
         }
         Ok(())
     }
