@@ -6,7 +6,7 @@
 //!   [regular expression language]: https://tools.ietf.org/html/rfc4880#section-8
 //!   [trust signatures]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
 //!
-//! Compared with most regular expression lanugages, OpenPGP's is
+//! Compared with most regular expression languages, OpenPGP's is
 //! quite simple.  In particular, it only includes the following
 //! features:
 //!
@@ -869,7 +869,7 @@ impl RegexSet {
     ///
     /// This can be used to optimize the evaluation of scoping rules
     /// along a path: if a `RegexSet` matches everything, then it
-    /// doesn't further contrain the path.
+    /// doesn't further constrain the path.
     pub fn everything() -> Result<Self>
     {
         Ok(Self {
@@ -1986,6 +1986,24 @@ mod tests {
             (true, "c"),
         ]);
 
+        // Make sure - is recognized as an atom when it is not part of
+        // a range.  That is: a-z matches a or - or z, but it doesn't
+        // match b (it's not a range).
+        a("a-z", &[
+            (true, "a-z"),
+            (false, "a"),
+            (false, "-"),
+            (false, "z"),
+            (false, "c"),
+        ]);
+
+        a("a|-|z", &[
+            (true, "a"),
+            (true, "-"),
+            (true, "z"),
+            (false, "c"),
+        ]);
+
         Ok(())
     }
 
@@ -2036,4 +2054,22 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn regex_set_sequoia() -> Result<()> {
+        let re = RegexSet::new(&["<[^>]+[@.]sequoia-pgp\\.org>$"])?;
+        dbg!(&re);
+        assert!(re.is_match("<justus@sequoia-pgp.org>"));
+        assert!(!re.is_match("<justus@gnupg.org>"));
+        Ok(())
+    }
+
+    #[test]
+    fn regex_set_sequoia_nodash() -> Result<()> {
+        let re = RegexSet::new(&["<[^>]+[@.]sequoiapgp\\.org>$"])?;
+        dbg!(&re);
+        assert!(re.is_match("<justus@sequoiapgp.org>"));
+        assert!(!re.is_match("<justus@gnupg.org>"));
+        Ok(())
+    }
 }
